#include "stdafx.h"
#include "usbm3069.h"
#include <math.h>

/*****************************************************************************
  TW_STATUS SetPwmPulse(TW_HANDLE hDev, long CH, 
                        long *pFrequency, long *pDuty)

  hDev       : foCX̃nh
  CH         : ݒ肷16rbg^C}̃`l
  pFrequency : [in]]̎g [out]ۂɐݒłg
  pDuty      : [in]]̃f[eB [out]ۂɐݒłf[eB

  16rbg^C}̎w`lPWMo͔g`̐ݒs܂B
  g[Hz][qAf[eB[%]P(0-99)Ŏw肵܂B
  g`̓NbN𕪎č邽߁AgAf[eBƂݒłl
  ܂B
*****************************************************************************/
TW_STATUS SetPwmPulse(TW_HANDLE hDev, long CH, long *pFrequency, long *pDuty)
{
  FT_STATUS ret;
  long freq;
  long HtoL, LtoH;
  long duty;

  if(!pFrequency || *pFrequency == 0) return TW_INVALID_ARGS;
  if(*pFrequency > 12500000) *pFrequency = 12500000; //16rbg^C}̎g12.5MHz܂

  //f[eBZbg
  if(pDuty) duty = *pDuty;
  else duty = 50;            //w肪ꍇ50%

  if(duty >= 100) duty = 99; //100%ɂ͂łȂ
  //duty0̏ꍇɂHtoLLtoH菬
  if(duty == 0){
    return USBM_TimerSetPulse(hDev, CH, 65535, 65534);
  }

  BYTE Clk = USBM_TCLK25000; //CLK=25MHz
  HtoL = 25000000 / *pFrequency;
  freq = 25000000 / HtoL; //ۂ̎gvZ

  if(HtoL > 65536){
    Clk = USBM_TCLK12500; //CLK=12.5MHz
    HtoL = 12500000 / *pFrequency;
    freq = 1250000 / HtoL;
  }
  if(HtoL > 65536){
    Clk = USBM_TCLK6250; //CLK=6.25MHz
    HtoL = 6250000 / *pFrequency;
    freq = 6250000 / HtoL;
  }
  if(HtoL > 65536){
    Clk = USBM_TCLK3125; //CLK=3125kHz
    HtoL = 3125000 / *pFrequency;
    if(HtoL > 65536) HtoL = 65536;
    freq = 3125000 / HtoL;
  }
  
  LtoH = ((100 - duty) * HtoL + 50) / 100; //"Lo""Hi"̃^C~OvZ(_ȉľܓ)
  //ݒs\Ȕ͈͂̏ꍇC
  if(LtoH == 0) LtoH = 1;
  if(pDuty) *pDuty = (long)((100.0 - LtoH * 100 / HtoL) + 0.5); //ݒłf[eBvZ

  HtoL--;
  LtoH--;

  if(ret = USBM_TimerSetClk(hDev, CH, Clk)) return ret;
  if(ret = USBM_TimerSetPulse(hDev, CH, (WORD)LtoH, (WORD)HtoL)) return ret;

  *pFrequency = freq;

  return ret;
}


//DARo[^ւ̓]gݒ肵܂B
TW_STATUS DA_SetFrequency(FT_HANDLE hDev, long CH, long *pFrequency)
{
  FT_STATUS ret;
  long freq;

  if(!pFrequency || *pFrequency == 0) return TW_INVALID_ARGS;
  
  //DA̕ϊ1MHz܂
  //̐DMA̓]TCŇEɂ̂łB
  //1`l̓̏ꍇ2MHz܂ŉ\łB
  if(*pFrequency > 1000000) *pFrequency = 1000000;

  long Clk = USBM_TCLK25000; //CLK=25MHz
  long Cmp = 25000000 / *pFrequency;
  freq = 25000000 / Cmp; //ۂ̒lvZ

  if(Cmp > 65536){
    Clk = USBM_TCLK12500; //CLK=12.5MHz
    Cmp = 12500000 / *pFrequency;
    freq = 1250000 / Cmp;
  }
  if(Cmp > 65536){
    Clk = USBM_TCLK6250; //CLK=6.25MHz
    Cmp = 6250000 / *pFrequency;
    freq = 6250000 / Cmp;
  }
  if(Cmp > 65536){
    Clk = USBM_TCLK3125; //CLK=3125kHz
    Cmp = 3125000 / *pFrequency;
    if(Cmp > 65536) Cmp = 65536;
    freq = 3125000 / Cmp;
  }
  Cmp--;

  //ݒ
  if(ret = USBM_DASetCycle(hDev, CH, (WORD)Cmp, Clk)) return ret;
  if(ret) return ret;

  *pFrequency = freq;
  return TW_OK;
}


//DARo[^̏o͔g`ݒ肵܂B
TW_STATUS DA_GenerateWave(TW_HANDLE hDev, long CH, long WaveType, long *pFrequency)
{
  FT_STATUS ret;
  BYTE Pattern[100];
  DWORD BuffAddress;
  const double pi = 3.1415926535;
  double dTmp;
  int i;

  CH &= 0x01;  //`l0܂1

  if(*pFrequency > 10000) *pFrequency = 10000; //10kHz܂
  *pFrequency *= 100;
  if(ret = DA_SetFrequency(hDev, CH, pFrequency)) return ret;
  
  switch(WaveType){
  case 1:  //`g
    for(i=0;i<50;i++) Pattern[i] = 0;
    for(;i<100;i++) Pattern[i] = 255;
    break;
  case 2:  //̂g
    for(i=0;i<100;i++){
      dTmp = 255.0 * i / 100;
      Pattern[i] = (BYTE)dTmp + 25;
    }
    break;
  case 3:  //Opg
    for(i=0;i<50;i++){
      dTmp = 255.0 * i / 50;
      Pattern[i] = (BYTE)dTmp; //ItZbg𒲐
    }
    for(;i<100;i++){
      dTmp = 255.0 * (100 - i) / 50;
      Pattern[i] = (BYTE)dTmp;
    }
    break;
  case 4:  //g
    for(i=0;i<100;i++){
      dTmp = sin(2 * pi * i / 100); //100f[^1
      dTmp *= 127;
      Pattern[i] = (BYTE)(dTmp + 128); //ItZbg𒲐
    }
    break;
  default: //~
    if(ret = USBM_DAStop(hDev, 0x01 << CH)) return ret; //w`l~
    return USBM_PortWrite8(hDev, USBM_DA0 + CH, 0);     //w`l̏o͂0ɐݒ
  }

  BuffAddress = USBM_USER_AREA + 8192 - (CH * 100);  //[U[GǍ납200oCggp
  USBM_DAStop(hDev, 0x01 << CH);
  if(ret = USBM_PortBWrite(hDev, BuffAddress, Pattern, 100, 1, FALSE)) return ret;
  if(ret = USBM_DASetParm(hDev, CH, BuffAddress, 100, TRUE)) return ret;

  *pFrequency = (long)((double)*pFrequency / 100 + 0.5);
  return USBM_DAStart(hDev, 0x01 << CH);  //Y`l]Jn
}


